﻿using System;
using System.IO;
using System.Runtime.InteropServices;

namespace CSV2HST
{
    ///<summary>
    /// Helper class for marshaling generic structures. Structures have to have the
    /// attribute: [<c>StructLayout</c>(<see cref="LayoutKind"/>.Sequential, Pack = 1)]
    ///</summary>
    public static class MarshalHelper
    {

        ///<summary>    
        /// Read a structure of type T from stream    
        ///</summary>    
        ///<param name="fs"></param>    
        ///<typeparam name="T"></typeparam>    
        ///<returns></returns>    
        public static T Read<T>(Stream fs)
        {
            byte[] buffer = new byte[Marshal.SizeOf(typeof(T))];
            fs.Read(buffer, 0, Marshal.SizeOf(typeof(T)));
            return Read<T>(buffer);
        }

        ///<summary>    
        /// Read a structure of type T from a byte buffer    
        ///</summary>    
        ///<param name="buffer"></param>    
        ///<typeparam name="T"></typeparam>    
        ///<returns></returns>    
        public static T Read<T>(byte[] buffer)
        {
            GCHandle handle = GCHandle.Alloc(buffer, GCHandleType.Pinned);
            try
            {
                return (T)Marshal.PtrToStructure(handle.AddrOfPinnedObject(), typeof(T));
            }
            finally
            {
                handle.Free();
            }
        }

        ///<summary>    
        /// Write a structure of type T to stream    
        ///</summary>    
        ///<param name="fs"></param>    
        ///<param name="t"></param>    
        ///<typeparam name="T"></typeparam>    
        public static void Write<T>(Stream fs, T t)
        {
            int tsize = Marshal.SizeOf(typeof(T));
            byte[] buffer = new byte[tsize];
            Write(buffer, t);
            fs.Write(buffer, 0, tsize);
        }

        ///<summary>    
        /// Write a structure of type T to a byte buffer    
        ///</summary>    
        ///<param name="buffer"></param>    
        ///<param name="t"></param>    
        ///<typeparam name="T"></typeparam>    
        public static void Write<T>(byte[] buffer, T t)
        {
            GCHandle handle = GCHandle.Alloc(buffer, GCHandleType.Pinned);
            try
            {
                Marshal.StructureToPtr(t, handle.AddrOfPinnedObject(), false);
            }
            finally
            {
                handle.Free();
            }
        }
    }
}